package cn.com.ty.lift.enterprise.custom.service.impl;

import cn.com.ty.lift.enterprise.custom.dao.entity.PropertyCompany;
import cn.com.ty.lift.enterprise.custom.dao.entity.PropertyContact;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyReq;
import cn.com.ty.lift.enterprise.custom.dao.mapper.PropertyCompanyMapper;
import cn.com.ty.lift.enterprise.custom.service.PropertyCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;


/**
 * <p>
 * (甲方公司)客户表 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-11-11
 */
@Service
@Transactional
@AllArgsConstructor
public class PropertyCompanyServiceImpl extends ServiceImpl<PropertyCompanyMapper, PropertyCompany> implements PropertyCompanyService {
    private final PropertyContactServiceImpl propertyContactService;
    private final RestTemplate restTemplate;

    public RestResponse delete(Long id) {
        Map<String, Object> map = new HashMap<>();
        map.put("projectStatus", 2);
        map.put("ppCompanyId", id);
        map.put("pageNum", 1);
        map.put("pageSize", 1);
        ResponseEntity<HashMap> objectResponseEntity = restTemplate.postForEntity("http://132.232.206.88:20227/project/companyProjectList", map, HashMap.class);
        String statusCode = (String) objectResponseEntity.getBody().get("statusCode");
        if (statusCode.equals("1")) {
            //就不删除
            return RestResponse.ok(null, "0", "有正在服务的关联项目，无法删除");
        }
        //如果有这项，逻辑删除
        PropertyCompany one = this.getById(id);
        if (null == one) {
            return RestResponse.ok(null, "0", "删除失败");
        }

        one.setStatus(0);
        this.updateById(one);
        this.deleteForContact(id);
        return RestResponse.ok(null, "1", "删除成功");
    }

    public boolean deleteForContact(Long id) {
        PropertyContact propertyContact = new PropertyContact();
        propertyContact.setStatus(0);
        return propertyContactService.update(propertyContact,
                new QueryWrapper<PropertyContact>()
                        .eq("pp_company_id", id));
    }

    public RestResponse propertyCompanies(PropertyCompanyReq propertyCompany) {
        Page<PropertyCompany> page = new Page<>(propertyCompany.getPageNum(), propertyCompany.getPageSize());
        //根据请求参数查询客户列表
        IPage<PropertyCompany> records = this
                .page(page, new QueryWrapper<PropertyCompany>()
                        .select("id", "mt_company_id", "name", "mailing_address", "telephone", "status")
                        .eq("mt_company_id", propertyCompany.getMtCompanyId())
                        .like(propertyCompany.getName() != null, "name", propertyCompany.getName())
                        .like(propertyCompany.getAddress() != null, "address", propertyCompany.getAddress())
                        .like(propertyCompany.getTelephone() != null, "telephone", propertyCompany.getTelephone())
                        .like(propertyCompany.getMailingAddress() != null, "mailing_address", propertyCompany.getMailingAddress()));
        if (records.getRecords().isEmpty()) {
            return RestResponse.ok(null, "9", "无数据");
        }
        return RestResponse.ok(records, "1", "查询成功");
    }

    public void export() {
        ExcelWriter writer = ExcelUtil.getWriter("客户列表.xls");
        //设置列别名
        paramMap.forEach(writer::addHeaderAlias);
        writer.write(this.list(), true);
        writer.autoSizeColumnAll();
        writer.close();
    }

    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("id", "客户id");
        put("mtCompanyId", "维保公司id");
        put("code", "编号");
        put("name", "名称");
        put("address", "地址");
        put("mailingAddress", "通讯地址");
        put("actualNum", "系统台量");
        put("address", "项目地址");
        put("startDate", "开始时间");
        put("endDate", "结束时间");
    }};
}