package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.system.user.dao.entity.UserAccount;
import cn.com.ty.lift.system.user.dao.entity.UserLogin;
import cn.com.ty.lift.system.user.dao.entity.model.UserRequest;
import cn.com.ty.lift.system.user.dao.entity.model.UserResponse;
import cn.com.ty.lift.system.user.service.ILoginService;
import cn.com.ty.lift.system.user.service.IUserAccountService;
import cn.com.ty.lift.system.user.service.IUserLoginService;
import cn.com.ty.lift.system.user.service.IUserService;
import cn.com.ty.lift.system.utils.PasswordUtils;
import cn.com.ty.lift.system.utils.ProjectUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.json.JSONUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * @author huangyuan
 * @date 2019-11-29
 * @description 用户登录接口
 */
@Service
public class LoginService implements ILoginService {

    @Autowired
    private IUserService userService;

    @Autowired
    private IUserAccountService userAccountService;

    @Resource
    private RedisTemplate redisTemplate;

    @Autowired
    private IUserLoginService userLoginService;

    @Override
    public RestResponse register(UserRequest userRequest) {
        //判断手机号是否为空
        if(StringUtils.isBlank(userRequest.getMobile())){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "手机号为空");
        }
        //判断密码是否为空
        if(StringUtils.isBlank(userRequest.getPassword())){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "密码为空");
        }

        //保存用户信息,并返回用户账户信息
        UserAccount userAccount = userService.saveUserInfo(userRequest);
        if(userAccount != null){
            return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "注册成功");
        }
        return RestResponse.error(ApiConstants.RESULT_ERROR, "注册失败");
    }

    @Override
    public RestResponse login(HttpServletRequest request, UserRequest userRequest) {
        String password = userRequest.getPassword();
        if(StringUtils.isBlank(userRequest.getMobile())){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "手机号为空无法注册");
        }
        if(StringUtils.isBlank(password)){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "用户密码为空无法注册");
        }
        //通过手机号获取用户账户信息
        UserAccount userAccount = userAccountService.getByMobile(userRequest.getMobile());
        //验证手机号是否存在
        if(userAccount == null){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "用户名不存在");
        }
        //校验密码是否正确
        String inputPassword = PasswordUtils.generatePassword(password, userAccount.getSalt());
        if(!inputPassword.equals(userAccount.getPassword())){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "密码输入有误");
        }
        UserResponse userResponse = userService.getLoginUserInfo(userAccount.getUserId());
        //将用户登录的信息放入redis中,作为缓存处理
        redisTemplate.opsForValue().set(userResponse.getToken(), JSONUtil.toJsonPrettyStr(userAccount));
        UserLogin userLogin = new UserLogin();
        userLogin.setUserId(userAccount.getUserId());
        userLogin.setIp(ProjectUtils.getIpAddress(request));
        //添加登录记录
        userLoginService.save(userLogin);
        return RestResponse.ok(userResponse,ApiConstants.RESULT_SUCCESS, "登录成功");
    }

    @Override
    public RestResponse resetPassword(UserRequest userRequest) {
        UserAccount userAccount = userAccountService.getByMobile(userRequest.getMobile());
        if(userAccount == null) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "手机号尚未注册");
        }
        String salt = userAccount.getSalt();
        //新密码加密
        String newPassword = PasswordUtils.generatePassword(userRequest.getPassword(), salt);
        userAccount.setPassword(newPassword);
        userAccountService.updateById(userAccount);
        return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "重置密码成功");
    }
}
